#![feature(proc_macro_hygiene, decl_macro)]

#[macro_use]
extern crate rocket;
use rand::Rng;
use rocket::http::{ContentType, Cookie, Cookies, Status};
use rocket::request;
use rocket::request::{Form, FromRequest, Request};
use rocket::response::{NamedFile, Redirect};
use rocket_contrib::templates::Template;
use rocket_upload::MultipartDatas;
use serde::Deserialize;
use std::collections::HashMap;
use std::env;
use std::fs;
use std::path::Path;
use std::path::PathBuf;

#[macro_use]
extern crate lazy_static;

lazy_static! {
    static ref FSERVE_PASSWD: String =
        env::var("FSERVE_PASSWD").unwrap_or_else(|_| String::from(""));
    static ref DIR: PathBuf = Path::new(
        env::var("FSERVE_DIR")
            .unwrap_or_else(|_| String::from(""))
            .as_str()
    )
    .to_path_buf();
}

struct AdminUser;

impl<'a, 'r> FromRequest<'a, 'r> for AdminUser {
    type Error = ();

    fn from_request(request: &'a Request<'r>) -> request::Outcome<Self, Self::Error> {
        match request.cookies().get("key") {
            Some(key) => {
                if key.value() == FSERVE_PASSWD.as_str() {
                    request::Outcome::Success(AdminUser)
                } else {
                    request::Outcome::Failure((Status::BadRequest, ()))
                }
            }

            None => request::Outcome::Failure((Status::BadRequest, ())),
        }
    }
}

#[get("/f/<path..>")]
fn get_file(path: PathBuf) -> Option<NamedFile> {
    let dir = DIR.as_path().join(path);
    rocket::response::NamedFile::open(dir.as_path()).ok()
}

fn upload(data: MultipartDatas) -> Vec<String> {
    let mut rng = rand::thread_rng();
    let mut v = Vec::new();
    for f in data.files {
        let mut hex_str = format!("{:016x}", rng.gen::<u64>());
        while Path::new(&format!("{}/{}", DIR.to_str().unwrap(), hex_str)).exists() {
            hex_str = format!("{:016x}", rng.gen::<u64>());
        }
        let dirloc = &format!("{}/{}", DIR.to_str().unwrap(), hex_str);
        fs::create_dir_all(Path::new(dirloc)).unwrap();
        let dir = Path::new(dirloc);
        v.push(format!("/{}/{}", hex_str, f.filename));
        f.persist(dir);
    }
    v
}

#[post("/upload", data = "<data>")]
fn upload_file(_content_type: &ContentType, data: MultipartDatas, _guard: AdminUser) -> Redirect {
    let resp = upload(data);
    Redirect::to(uri!(get_file: Path::new(&resp[0])))
}

#[get("/up")]
fn up(_guard: AdminUser) -> Template {
    let map: HashMap<String, String> = HashMap::new();
    Template::render("up", map)
}

#[derive(Debug, FromForm, Deserialize)]
struct GenericUser {
    key: String,
}

#[post("/login", data = "<form>")]
fn login(form: Form<GenericUser>, mut cookies: Cookies) -> Redirect {
    let key = form.into_inner().key;
    let mut cook = Cookie::new("key", key);
    cook.set_http_only(true);
    cook.set_secure(true);
    cookies.add(cook);
    Redirect::to("/up")
}

#[get("/")]
fn root() -> Template {
    let map: HashMap<String, String> = HashMap::new();
    Template::render("home", map)
}

fn main() {
    rocket::ignite()
        .attach(Template::fairing())
        .mount("/", routes![get_file, upload_file, root, login, up])
        .launch();
}
